/**
 * Created by Tobi Stockinger on 25/11/14.
 */

var API = function(){

    // these backend routes are known to the API
    const routes = {
        service : '/service',
        // actually this will be compiled to /service/suggestions
        suggestions : '/suggestions'
    };

    // voting methods
    var methods = {
        up : 'up',
        down : 'down'
    };


    /**
     * generic vo
     * @param suggestionId integer identifying the suggestion
     * @param method {up,down}
     * @returns {$.Deferred} that can be used to attache a .done() callback. Should be done in the calling function.
     */
    var vote = function(suggestionId, method){
        return $.post(
            routes.service,
            {
                suggestionId : suggestionId,
                method : method
            });
    };



    return {
        /**
         * shorthand function for voting with methods.up
         * @param suggestionId
         * @returns {$.Deferred}
         */
        votePositive: function(suggestionId){
            return vote(suggestionId,methods.up);
        },
        /**
         * shorthand function for voting with methods.down
         * @param suggestionId
         * @returns {$.Deferred}
         */
        voteNegative: function(suggestionId){
            return vote(suggestionId,methods.down);
        },
        /**
         * returns a list of all suggestions to the callback(data)
         * @param callback
         */
        getSuggestions : function(callback){
            $.get(routes.service + routes.suggestions,callback)
        },
        /**
         * POSTs a new suggestion to the backend
         * @param text
         * @param callback
         */
        suggest: function(text,callback){
            $.post(routes.service + routes.suggestions,{suggestionText : text},callback)
        }
    };
};
