/**
 * Created by Tobi Stockinger on 29/11/14.
 */



$(document).ready(function () {

    const votingOptions = {
        POSITIVE: 'up',
        NEGATIVE: 'down'
    };

    // this will hold our suggestions;
    var hasLoaded;

    // the API encapsulates all async calls
    var api = new API();

    var suggestionForm = $('#postSuggestionForm');

    // the score is simply upVotes - downVotes;
    var getScore = function (suggestion) {
        return suggestion ? suggestion.votes.up - suggestion.votes.down : 0;
    };

    var initButtons = function () {
        var votingButtons = $('.vote');
        // we transform all .vote divs into buttons
        // then we add click handlers that trigger
        // async api calls
        votingButtons.each(function (index, button) {
            button = $(button);

            var buttonType = button.hasClass('up') ? votingOptions.POSITIVE : votingOptions.NEGATIVE;
            var icon = buttonType == votingOptions.POSITIVE ? 'ui-icon-triangle-1-n' : 'ui-icon-triangle-1-s';

            // generateClickHandler is a closure!
            // the returned function depends on id even though it is already garbage.
            var generateClickHandler = function (id) {
                return function () {
                    var deferred;
                    switch (buttonType) {
                        case votingOptions.POSITIVE:
                            deferred = api.votePositive(id);
                            break;
                        case votingOptions.NEGATIVE:
                            deferred = api.voteNegative(id);
                            break;
                    }
                    deferred.done(votingResponseHandler);
                }
            };

            button.button({
                icons: {
                    primary: icon
                }
            }).unbind('click').click(generateClickHandler(button.parent().data().suggestion));
        });
    };
    /**
     * sorts the displayed suggestions
     * A small animation gives visual feedback for the sorting routine
     */
    var updateUI = function () {
        var container = $('#container');

        var suggestionDivs = container.find('.suggestion');
        var divArray = suggestionDivs.toArray();

        divArray.sort(function (a, b) {
            var suggestionA = $(a).data('data');
            var suggestionB = $(b).data('data')
            var scoreA = getScore(suggestionA);
            var scoreB = getScore(suggestionB);
            if (!suggestionA || !suggestionB) {
                console.log(a);
                console.log(b);
            }
            var result = scoreB - scoreA;
            return result;
        });

        $.each(suggestionDivs,function(index,div){
            if(!$(div).is($(divArray[index]))){
                $(div).fadeOut();
                $(divArray[index]).fadeOut();
            }
        });

        // reorder by shortly (100ms) hiding the content

        $(divArray).appendTo('#suggestions');
        $(divArray).fadeIn();


        // make all buttons clickable
        initButtons();

    };

    /**
     * @callback for api deferred.done
     * @param response
     */
    var votingResponseHandler = function (response) {
        var suggestion;
        var container, upButton, downButton;

        // only proceed if the query was successful
        if (response && response.status && response.status == 200) {
            suggestion = response.suggestion;


            if (suggestion) {
                container = $('div.suggestion[data-suggestion=' + suggestion.id + ']');
                upButton = container.find('.vote.up');
                downButton = container.find('.vote.down');

                upButton.button("option", "label", suggestion.votes.up);
                downButton.button("option", "label", suggestion.votes.down);


                // now let's mutually toggle the button states
                // according to the user's own vote
                if (suggestion.ownVote == votingOptions.POSITIVE) {
                    upButton.button("disable");
                    downButton.button("enable");
                }
                else if (suggestion.ownVote == votingOptions.NEGATIVE) {
                    upButton.button("enable");
                    downButton.button("disable");
                }

                container.data('data',suggestion);

                updateUI();
            }
            else {
                console.log("suggestion is empty");
            }
        }
        else {
            console.log(response);
        }
    };

    // this is mainly responsible for two things:
    // 1. - disabling the buttons for the suggestions that the user has already voted vor
    // 2. - re-ordering the suggestions so that the ones with the highest scores appear first.
    var suggestionLoadCallback = function (data) {

        var suggestions = data.suggestions;


        initButtons();

        $.each(suggestions, function (idx, suggestion) {
            var container = $('div.suggestion[data-suggestion=' + suggestion.id + ']');
            container.data('data', suggestion);
            if (suggestion.ownVote) {
                console.log("disabling : " + suggestion.id + " " + suggestion.ownVote);
                container.find('.vote.' + suggestion.ownVote).button("disable");
            }
        });

        // we only sort the data if the suggestions were requested _again_
        // because we assume that we get them sorted anyway
        if (!hasLoaded) {
            updateUI();
        }
        hasLoaded = true;
    };


    // now let's update the UI with an API call.
    api.getSuggestions(suggestionLoadCallback);


    /**
     * let's send the input from the suggestion field
     * to the backend.
     */
    suggestionForm.submit(function (e) {
        e.preventDefault();
        var input = $(this).find('#suggestionText');
        var text = input.val();
        api.suggest(text, function (data) {
            var submittedSuggestion;
            // only proceed if the data returns an id that we can process
            if (data.suggestions.id) {
                submittedSuggestion = data.suggestions;

                // let's construct a div that looks just like the other suggestion divs
                var container = $('#container #suggestions');
                var suggestionDiv = $('<div />').addClass('suggestion').data('suggestion', submittedSuggestion.id);
                suggestionDiv.data('data',submittedSuggestion);
                suggestionDiv.attr('data-suggestion',submittedSuggestion.id);
                suggestionDiv.append($('<div />').addClass('text').html(submittedSuggestion.text));
                suggestionDiv.append($('<div />').addClass('vote up').html(submittedSuggestion.votes.up));
                suggestionDiv.append($('<div />').addClass('vote down').html(submittedSuggestion.votes.down));
                suggestionDiv.append($('<div />').addClass('clear'));
                suggestionDiv.appendTo(container);
                suggestionLoadCallback({suggestions:[data.suggestions]});
            }
        });
        // reset the input
        input.val(null);
    });

});